/**
 * @license MIT
 * author: undefined
 * jenga-ui v0.6.0
 * Released under the MIT license.
 */

import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import styled from 'styled-components';
import { useDOMRef } from '@react-spectrum/utils';
import { DismissButton } from '@react-aria/overlays';
import { forwardRef } from 'react';
import { useDialog } from '@react-aria/dialog';
import { useMessageFormatter } from '@react-aria/i18n';
import { X } from '@jengaicons/react';
import FocusLock from 'react-focus-lock';
import { BASE_STYLES, BLOCK_STYLES, DIMENSION_STYLES, extractStyles, tasty, } from '../../../tasty';
import { mergeProps, SlotProvider } from '../../../utils/react';
import { Button } from '../../actions';
import { useOpenTransitionContext } from '../Modal/OpenTransition';
import { useDialogContext } from './context';
const STYLES_LIST = [...BASE_STYLES, ...DIMENSION_STYLES, ...BLOCK_STYLES];
const DialogElement = tasty({
    as: 'section',
    styles: {
        pointerEvents: 'auto',
        position: 'relative',
        display: 'flex',
        placeItems: 'stretch',
        placeContent: 'stretch',
        width: {
            '': '288px @dialog-size 90vw',
            '[data-type="fullscreen"]': '90vw 90vw',
            '[data-type="fullscreenTakeover"]': '100vw 100vw',
            '[data-type="panel"]': '100vw 100vw',
        },
        height: {
            '': 'max 90vh',
            '[data-type="fullscreenTakeover"] | [data-type="panel"]': 'max 100vh',
        },
        gap: 0,
        flow: 'column',
        radius: {
            '': '2r',
            '[data-type="tray"]': '2r top',
            '[data-type="fullscreenTakeover"] | [data-type="panel"]': '0r',
        },
        fill: '#white',
        shadow: {
            '': '0 20px 30px #shadow',
            '[data-type="popover"]': '0px 4px 16px #shadow',
        },
        transform: {
            '': false,
            '[data-type="modal"]': 'translate(0, ((50vh - 50%) / -3))',
        },
        placeSelf: 'stretch',
        '@dialog-heading-padding-v': {
            '': '2x',
            '[data-type="popover"]': '1x',
        },
        '@dialog-content-padding-v': {
            '': '3x',
            '[data-type="popover"]': '2x',
        },
        '@dialog-padding-h': {
            '': '3x',
            '[data-type="popover"]': '2x',
        },
        '@dialog-footer-v': {
            '': '2x',
            '[data-type="popover"]': '1x',
        },
        '@dialog-content-gap': '3x',
    },
});
const StyledFocusLock = styled(FocusLock) `
  display: contents;
`;
const CLOSE_BUTTON_STYLES = {
    display: 'flex',
    position: 'absolute',
    top: '1x',
    right: '1x',
    width: '5x',
    height: '5x',
    placeContent: 'center',
};
const sizeMap = {
    S: 'small',
    M: 'medium',
    L: 'large',
};
const sizePxMap = {
    small: 360,
    medium: 479,
    large: 798,
};
const intlMessages = {
    'en-US': {
        dismiss: 'Dismiss',
        alert: 'Alert',
    },
};
/**
 * Dialogs are windows containing contextual information, tasks, or workflows that appear over the user interface.
 * Depending on the kind of Dialog, further interactions may be blocked until the Dialog is acknowledged.
 */
export const Dialog = forwardRef(function Dialog(props, ref) {
    const transitionContext = useOpenTransitionContext();
    const isEntered = transitionContext?.transitionState === 'entered';
    return (_jsx(StyledFocusLock, { returnFocus: true, disabled: !isEntered, children: _jsx(DialogContent, { ...props, ref: ref }, "content") }, void 0));
});
const DialogContent = forwardRef(function DialogContent(props, ref) {
    let { type = 'modal', ...contextProps } = useDialogContext();
    let { qa, children, size = 'S', isDismissable = contextProps.isDismissable, onDismiss = contextProps.onClose, closeIcon, closeButtonStyles, ...otherProps } = props;
    size = sizeMap[size.toUpperCase()] || size;
    const styles = extractStyles(otherProps, STYLES_LIST);
    let formatMessage = useMessageFormatter(intlMessages);
    let domRef = useDOMRef(ref);
    let { dialogProps, titleProps } = useDialog(mergeProps(contextProps, props), domRef);
    // If rendered in a popover or tray there won't be a visible dismiss button,
    // so we render a hidden one for screen readers.
    let dismissButton;
    if (type === 'popover' || type === 'tray') {
        dismissButton = _jsx(DismissButton, { onDismiss: onDismiss }, void 0);
    }
    // let hasHeader = useHasChild('[data-id="Header"]', domRef);
    // let hasFooter = useHasChild('[data-id="Footer"]', domRef);
    let slots = {
        heading: {
            level: 2,
            preset: 'h4',
            ...titleProps,
        },
        content: {
            styles: {
                flexGrow: 1,
                padding: '@dialog-content-padding-v @dialog-padding-h',
                gap: '@dialog-content-gap',
                height: {
                    '': 'max (100% - (2 * @dialog-content-padding-v))',
                    ':last-child': 'max (100% - @dialog-content-padding-v)',
                },
            },
        },
        header: {
            ellipsis: true,
            styles: {
                display: 'flex',
                flow: 'row',
                gap: '1x',
                placeItems: 'baseline stretch',
                placeContent: 'space-between',
                padding: `@dialog-heading-padding-v ${isDismissable ? '(@dialog-padding-h + 4x)' : '@dialog-padding-h'} @dialog-heading-padding-v @dialog-padding-h`,
                border: 'bottom',
            },
        },
        footer: {
            styles: {
                display: 'flex',
                gap: '1x',
                flow: 'row',
                placeItems: 'baseline stretch',
                placeContent: 'space-between',
                padding: '@dialog-footer-v @dialog-padding-h',
            },
        },
    };
    return (_jsxs(DialogElement, { ref: domRef, "data-id": "Dialog", "data-qa": qa || 'Dialog', styles: styles, as: "section", ...dialogProps, mods: { dismissable: isDismissable }, style: { '--dialog-size': `${sizePxMap[size] || 288}px` }, "data-type": type, "data-size": size, children: [dismissButton, _jsxs(SlotProvider, { slots: slots, children: [isDismissable && (_jsx(Button, { qa: "ModalCloseButton", type: "neutral", styles: CLOSE_BUTTON_STYLES, icon: closeIcon || _jsx(X, {}, void 0), label: formatMessage('dismiss'), onPress: () => onDismiss && onDismiss() }, void 0)), children] }, void 0)] }, void 0));
});

        