/**
 * @license MIT
 * author: undefined
 * jenga-ui v0.6.0
 * Released under the MIT license.
 */

import { jsx as _jsx } from "react/jsx-runtime";
import styledComponents, { createGlobalStyle } from 'styled-components';
import { forwardRef, useContext, useMemo } from 'react';
import { isValidElementType } from 'react-is';
import { BreakpointsContext } from './providers/BreakpointsProvider';
import { modAttrs } from './utils/modAttrs';
import { renderStyles } from './utils/renderStyles';
import { pointsToZones } from './utils/responsive';
import { BASE_STYLES } from './styles/list';
import { cacheWrapper } from './utils/cache-wrapper';
import { mergeStyles } from './utils/mergeStyles';
import { getDisplayName } from './utils/getDisplayName';
// Implementation
function tasty(Component, options) {
    if (typeof Component === 'string') {
        let selector = Component;
        let styles = options;
        let Element = createGlobalStyle `${(css) => css}`;
        const _StyleDeclarationComponent = ({ breakpoints, }) => {
            let contextBreakpoints = useContext(BreakpointsContext);
            breakpoints = breakpoints ?? contextBreakpoints;
            let css = useMemo(() => styles
                ? `\n{}${selector}{${renderStyles(styles, pointsToZones(breakpoints || contextBreakpoints))}}`
                : '', [breakpoints.join(',')]);
            return _jsx(Element, { css: css }, void 0);
        };
        _StyleDeclarationComponent.displayName = `TastyStyleDeclaration(${Component})`;
        return _StyleDeclarationComponent;
    }
    if (isValidElementType(Component)) {
        let { as: extendTag, element: extendElement, ...defaultProps } = options ?? {};
        let propsWithStyles = ['styles'].concat(Object.keys(defaultProps).filter((prop) => prop.endsWith('Styles')));
        let _WrappedComponent = forwardRef((props, ref) => {
            const { as, element, ...restProps } = props;
            const propsWithStylesValues = propsWithStyles.map((prop) => props[prop]);
            const mergedStylesMap = useMemo(() => {
                return propsWithStyles.reduce((map, prop) => {
                    if (restProps[prop] != null && defaultProps[prop] != null) {
                        map[prop] = mergeStyles(defaultProps[prop], restProps[prop]);
                    }
                    else {
                        map[prop] = restProps[prop] || defaultProps[prop];
                    }
                    return map;
                }, {});
            }, propsWithStylesValues);
            return (_jsx(Component, { ref: ref, ...defaultProps, ...restProps, ...mergedStylesMap, as: as ?? extendTag, element: element || extendElement }, void 0));
        });
        _WrappedComponent.displayName = `TastyWrappedComponent(${getDisplayName(Component, defaultProps.qa ?? extendTag ?? 'Anonymous')})`;
        return _WrappedComponent;
    }
    const tastyOptions = Component;
    let { as: originalAs = 'div', element: defaultElement, styles: defaultStyles, styleProps, variants, ...defaultProps } = tastyOptions;
    let _TastyComponent;
    if (variants) {
        /**
         * Create a tasty component for each variant.
         * Then return a component that receives a `variant` and chooses the right component.
         */
        const variantComponents = Object.keys(variants).reduce((map, variant) => {
            const variantStyles = variants?.[variant];
            map[variant] = tasty({
                as: originalAs,
                styles: mergeStyles(defaultStyles, variantStyles),
                styleProps,
                ...defaultProps,
            });
            return map;
        }, {});
        if (!variantComponents['default']) {
            variantComponents['default'] = tasty({
                as: originalAs,
                styles: defaultStyles,
                styleProps,
                ...defaultProps,
            });
        }
        // eslint-disable-next-line react/display-name
        _TastyComponent = forwardRef((allProps, ref) => {
            const { variant, ...restProps } = allProps;
            const Component = (variantComponents[variant] ||
                variantComponents['default']);
            return _jsx(Component, { ...restProps, ref: ref }, void 0);
        });
    }
    else {
        let Element = styledComponents[originalAs](({ css }) => css);
        /**
         * An additional optimization that allows to avoid rendering styles across various instances
         * of the same element if no custom styles are provided via `styles` prop or direct style props.
         */
        const renderDefaultStyles = cacheWrapper((breakpoints) => {
            return renderStyles(defaultStyles || {}, pointsToZones(breakpoints));
        });
        let { qa: defaultQa, qaVal: defaultQaVal, ...otherDefaultProps } = defaultProps ?? {};
        // eslint-disable-next-line react/display-name
        _TastyComponent = forwardRef((allProps, ref) => {
            let { as, styles, variant, breakpoints, mods, element, qa, qaVal, ...otherProps } = allProps;
            let propStyles = (styleProps
                ? styleProps.concat(BASE_STYLES)
                : BASE_STYLES).reduce((map, prop) => {
                if (prop in otherProps) {
                    map[prop] = otherProps[prop];
                    delete otherProps[prop];
                }
                return map;
            }, {});
            if (Object.keys(propStyles).length === 0) {
                propStyles = null;
            }
            if (!styles || (styles && Object.keys(styles).length === 0)) {
                styles = undefined;
            }
            const propStylesCacheKey = JSON.stringify(propStyles);
            const stylesCacheKey = useMemo(() => {
                return JSON.stringify(styles);
            }, [styles]);
            const useDefaultStyles = !propStyles && !styles;
            const styleCacheKey = useMemo(() => {
                return `${styles ? JSON.stringify(styles) : ''}.${propStyles ? JSON.stringify(propStyles) : ''}`;
            }, [propStylesCacheKey, stylesCacheKey]);
            let allStyles = useMemo(() => useDefaultStyles
                ? defaultStyles
                : mergeStyles(defaultStyles, styles, propStyles), [styleCacheKey]);
            let contextBreakpoints = useContext(BreakpointsContext);
            breakpoints = breakpoints ?? contextBreakpoints;
            let renderedStyles = useMemo(() => {
                return useDefaultStyles
                    ? renderDefaultStyles(breakpoints)
                    : renderStyles(allStyles, pointsToZones(breakpoints));
            }, [allStyles, breakpoints]);
            if (mods) {
                Object.assign(otherProps, modAttrs(mods));
            }
            return (_jsx(Element, { as: as ?? originalAs, "data-element": element || defaultElement, "data-qa": qa || defaultQa, "data-qaval": qaVal || defaultQaVal, ...otherDefaultProps, ...otherProps, ref: ref, css: renderedStyles }, void 0));
        });
    }
    _TastyComponent.displayName = `TastyComponent(${defaultProps.qa || originalAs})`;
    return _TastyComponent;
}
const Element = tasty({});
export { tasty, Element };

        